/*****************************************************************************
* | File      	:	Pico_LCD_0in96.c
* | Author      :   Luckfox team
* | Function    :   LCD driver
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-09-03
* | Info        :   
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
******************************************************************************/
#include "Pico_LCD_0in96.h"
#include <stdlib.h>		//itoa()
/*******************************************************************************
function:
	Hardware reset
*******************************************************************************/
static void PICO_LCD_0IN96_Reset(void)
{
	DEV_Delay_ms(200);
	DEV_Digital_Write(LCD_RST, 0);
	DEV_Delay_ms(200);
	DEV_Digital_Write(LCD_RST, 1);
	DEV_Delay_ms(200);
}

/*******************************************************************************
function:
		Write data and commands
*******************************************************************************/
static void PICO_LCD_0IN96_Write_Command(UBYTE data)	 
{	
	DEV_Digital_Write(LCD_DC, 0);
	DEV_SPI_WriteByte(data);
}

static void PICO_LCD_0IN96_WriteData_Byte(UBYTE data) 
{	
	DEV_Digital_Write(LCD_DC, 1);
	DEV_SPI_WriteByte(data);  
}  

void PICO_LCD_0IN96_WriteData_Word(UWORD data)
{
	DEV_Digital_Write(LCD_DC, 1);
	DEV_SPI_WriteByte((data>>8) & 0xff);
	DEV_SPI_WriteByte(data);
}	  

/******************************************************************************
function:	
		Common register initialization
******************************************************************************/
void PICO_LCD_0IN96_Init(void)
{
	PICO_LCD_0IN96_Reset();

	//************* Start Initial Sequence **********// 
	PICO_LCD_0IN96_Write_Command(0x11);//Sleep exit 
	DEV_Delay_ms (120);
	PICO_LCD_0IN96_Write_Command(0x21); 
	PICO_LCD_0IN96_Write_Command(0x21); 

	PICO_LCD_0IN96_Write_Command(0xB1); 
	PICO_LCD_0IN96_WriteData_Byte(0x05);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);

	PICO_LCD_0IN96_Write_Command(0xB2);
	PICO_LCD_0IN96_WriteData_Byte(0x05);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);

	PICO_LCD_0IN96_Write_Command(0xB3); 
	PICO_LCD_0IN96_WriteData_Byte(0x05);  
	PICO_LCD_0IN96_WriteData_Byte(0x3A);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);
	PICO_LCD_0IN96_WriteData_Byte(0x05);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);
	PICO_LCD_0IN96_WriteData_Byte(0x3A);

	PICO_LCD_0IN96_Write_Command(0xB4);
	PICO_LCD_0IN96_WriteData_Byte(0x03);

	PICO_LCD_0IN96_Write_Command(0xC0);
	PICO_LCD_0IN96_WriteData_Byte(0x62);
	PICO_LCD_0IN96_WriteData_Byte(0x02);
	PICO_LCD_0IN96_WriteData_Byte(0x04);

	PICO_LCD_0IN96_Write_Command(0xC1);
	PICO_LCD_0IN96_WriteData_Byte(0xC0);

	PICO_LCD_0IN96_Write_Command(0xC2);
	PICO_LCD_0IN96_WriteData_Byte(0x0D);
	PICO_LCD_0IN96_WriteData_Byte(0x00);

	PICO_LCD_0IN96_Write_Command(0xC3);
	PICO_LCD_0IN96_WriteData_Byte(0x8D);
	PICO_LCD_0IN96_WriteData_Byte(0x6A);   

	PICO_LCD_0IN96_Write_Command(0xC4);
	PICO_LCD_0IN96_WriteData_Byte(0x8D); 
	PICO_LCD_0IN96_WriteData_Byte(0xEE); 

	PICO_LCD_0IN96_Write_Command(0xC5);  /*VCOM*/
	PICO_LCD_0IN96_WriteData_Byte(0x0E);    

	PICO_LCD_0IN96_Write_Command(0xE0);
	PICO_LCD_0IN96_WriteData_Byte(0x10);
	PICO_LCD_0IN96_WriteData_Byte(0x0E);
	PICO_LCD_0IN96_WriteData_Byte(0x02);
	PICO_LCD_0IN96_WriteData_Byte(0x03);
	PICO_LCD_0IN96_WriteData_Byte(0x0E);
	PICO_LCD_0IN96_WriteData_Byte(0x07);
	PICO_LCD_0IN96_WriteData_Byte(0x02);
	PICO_LCD_0IN96_WriteData_Byte(0x07);
	PICO_LCD_0IN96_WriteData_Byte(0x0A);
	PICO_LCD_0IN96_WriteData_Byte(0x12);
	PICO_LCD_0IN96_WriteData_Byte(0x27);
	PICO_LCD_0IN96_WriteData_Byte(0x37);
	PICO_LCD_0IN96_WriteData_Byte(0x00);
	PICO_LCD_0IN96_WriteData_Byte(0x0D);
	PICO_LCD_0IN96_WriteData_Byte(0x0E);
	PICO_LCD_0IN96_WriteData_Byte(0x10);

	PICO_LCD_0IN96_Write_Command(0xE1);
	PICO_LCD_0IN96_WriteData_Byte(0x10);
	PICO_LCD_0IN96_WriteData_Byte(0x0E);
	PICO_LCD_0IN96_WriteData_Byte(0x03);
	PICO_LCD_0IN96_WriteData_Byte(0x03);
	PICO_LCD_0IN96_WriteData_Byte(0x0F);
	PICO_LCD_0IN96_WriteData_Byte(0x06);
	PICO_LCD_0IN96_WriteData_Byte(0x02);
	PICO_LCD_0IN96_WriteData_Byte(0x08);
	PICO_LCD_0IN96_WriteData_Byte(0x0A);
	PICO_LCD_0IN96_WriteData_Byte(0x13);
	PICO_LCD_0IN96_WriteData_Byte(0x26);
	PICO_LCD_0IN96_WriteData_Byte(0x36);
	PICO_LCD_0IN96_WriteData_Byte(0x00);
	PICO_LCD_0IN96_WriteData_Byte(0x0D);
	PICO_LCD_0IN96_WriteData_Byte(0x0E);
	PICO_LCD_0IN96_WriteData_Byte(0x10);

	PICO_LCD_0IN96_Write_Command(0x3A); 
	PICO_LCD_0IN96_WriteData_Byte(0x05);

	PICO_LCD_0IN96_Write_Command(0x36);
	PICO_LCD_0IN96_WriteData_Byte(0xA8);

	PICO_LCD_0IN96_Write_Command(0x29);
}

/*******************************************************************************
function:
	Setting backlight
parameter	:
	  value : Range 0~1000   Duty cycle is value/1000	
*******************************************************************************/
void PICO_LCD_0IN96_SetBackLight(UWORD Value)
{
	// DEV_Set_PWM(Value);
}

/******************************************************************************
function:	Set the cursor position
parameter	:
	  Xstart: 	Start UWORD x coordinate
	  Ystart:	Start UWORD y coordinate
	  Xend  :	End UWORD coordinates
	  Yend  :	End UWORD coordinatesen
******************************************************************************/
void PICO_LCD_0IN96_SetWindow(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD  Yend)
{ 
	Xstart = Xstart + 1;
	Xend = Xend + 1;
	Ystart = Ystart + 26;
	Yend = Yend+26;
	
	PICO_LCD_0IN96_Write_Command(0x2a);
	PICO_LCD_0IN96_WriteData_Byte(Xstart >> 8);
	PICO_LCD_0IN96_WriteData_Byte(Xstart);
	PICO_LCD_0IN96_WriteData_Byte(Xend >> 8);
	PICO_LCD_0IN96_WriteData_Byte(Xend );

	PICO_LCD_0IN96_Write_Command(0x2b);
	PICO_LCD_0IN96_WriteData_Byte(Ystart >> 8);
	PICO_LCD_0IN96_WriteData_Byte(Ystart);
	PICO_LCD_0IN96_WriteData_Byte(Yend >> 8);
	PICO_LCD_0IN96_WriteData_Byte(Yend);

	PICO_LCD_0IN96_Write_Command(0x2C);
}

/******************************************************************************
function:	Settings window
parameter	:
	  Xstart: 	Start UWORD x coordinate
	  Ystart:	Start UWORD y coordinate

******************************************************************************/
void PICO_LCD_0IN96_SetCursor(UWORD X, UWORD Y)
{ 
	X = X + 1;
	Y = Y + 26;
	
	PICO_LCD_0IN96_Write_Command(0x2a);
	PICO_LCD_0IN96_WriteData_Byte(X >> 8);
	PICO_LCD_0IN96_WriteData_Byte(X);
	PICO_LCD_0IN96_WriteData_Byte(X >> 8);
	PICO_LCD_0IN96_WriteData_Byte(X);

	PICO_LCD_0IN96_Write_Command(0x2b);
	PICO_LCD_0IN96_WriteData_Byte(Y >> 8);
	PICO_LCD_0IN96_WriteData_Byte(Y);
	PICO_LCD_0IN96_WriteData_Byte(Y >> 8);
	PICO_LCD_0IN96_WriteData_Byte(Y);

	PICO_LCD_0IN96_Write_Command(0x2C);
}

/******************************************************************************
function:	Clear screen function, refresh the screen to a certain color
parameter	:
	  Color :		The color you want to clear all the screen
******************************************************************************/
void PICO_LCD_0IN96_Clear(UWORD Color)
{
	unsigned int i,j;  
	Color = 0xffff;
    UWORD Image[PICO_LCD_0IN96_WIDTH*PICO_LCD_0IN96_HEIGHT];
    
    Color = ((Color<<8)&0xff00)|(Color>>8);
   
    for (j = 0; j < PICO_LCD_0IN96_HEIGHT*PICO_LCD_0IN96_WIDTH; j++) {
        Image[j] = Color;
    }
	PICO_LCD_0IN96_SetWindow(0, 0, PICO_LCD_0IN96_WIDTH-1, PICO_LCD_0IN96_HEIGHT-1);
	DEV_Digital_Write(LCD_DC, 1);
	for(i = 0; i < PICO_LCD_0IN96_HEIGHT; i++){
        DEV_SPI_Write_nByte((uint8_t *)&Image[i*PICO_LCD_0IN96_WIDTH], PICO_LCD_0IN96_WIDTH*2); 
	 }
}

/******************************************************************************
function:	Refresh a certain area to the same color
parameter	:
	  Xstart: 	Start UWORD x coordinate
	  Ystart:	Start UWORD y coordinate
	  Xend  :	End UWORD coordinates
	  Yend  :	End UWORD coordinates
	  color :	Set the color
******************************************************************************/
void PICO_LCD_0IN96_ClearWindow(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend,UWORD color)
{          
	UWORD i,j; 
	PICO_LCD_0IN96_SetWindow(Xstart, Ystart, Xend-1,Yend-1);
	for(i = Ystart; i <= Yend-1; i++){
		for(j = Xstart; j <= Xend-1; j++){
			PICO_LCD_0IN96_WriteData_Word(color);
		}
	}
}

/******************************************************************************
function: Draw a point
parameter	:
	    X	: 	Set the X coordinate
	    Y	:	Set the Y coordinate
	  Color :	Set the color
******************************************************************************/
void PICO_LCD_0IN96_DrawPaint(UWORD x, UWORD y, UWORD Color)
{
	PICO_LCD_0IN96_SetCursor(x, y);
	PICO_LCD_0IN96_WriteData_Word(Color); 	    
}

/******************************************************************************
function :	Sends the image buffer in RAM to displays
parameter:
******************************************************************************/
void PICO_LCD_0IN96_Display(UWORD *Image)
{
    UWORD j;
    PICO_LCD_0IN96_SetWindow(0, 0, PICO_LCD_0IN96_WIDTH-1, PICO_LCD_0IN96_HEIGHT-1);
    DEV_Digital_Write(LCD_DC, 1);
    for (j = 0; j < PICO_LCD_0IN96_HEIGHT; j++) {
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*PICO_LCD_0IN96_WIDTH], PICO_LCD_0IN96_WIDTH*2);
    }
}

void PICO_LCD_0IN96_DisplayWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend, UWORD *Image)
{
    // display
    UDOUBLE Addr = 0;

    UWORD j;
    PICO_LCD_0IN96_SetWindow(Xstart, Ystart, Xend-1 , Yend-1);
    DEV_Digital_Write(LCD_DC, 1);
    for (j = Ystart; j < Yend - 1; j++) {
        Addr = Xstart + j * PICO_LCD_0IN96_WIDTH ;
        DEV_SPI_Write_nByte((uint8_t *)&Image[Addr], (Xend-Xstart-1)*2);
    }
}

void  Handler_0IN96_LCD(int signo)
{
    //System Exit
    printf("\r\nHandler:Program stop\r\n");     
    DEV_ModuleExit();
	exit(0);
}
