/*****************************************************************************
* | File      	:   Pico_LCD_1in14.c
* | Author      :   Luckfox team
* | Function    :   Hardware underlying interface
* | Info        :
*                Used to shield the underlying layers of each master
*                and enhance portability
*----------------
* |	This version:   V1.0
* | Date        :   2023-09-03
* | Info        :   Basic version
*
******************************************************************************/
#include "Pico_LCD_1in14.h"
#include "DEV_Config.h"

#include <stdlib.h>		//itoa()
#include <stdio.h>

PICO_LCD_1IN14_ATTRIBUTES PICO_LCD_1IN14;


/******************************************************************************
function :	Hardware reset
parameter:
******************************************************************************/
static void PICO_LCD_1IN14_Reset(void)
{
    PICO_LCD_1IN14_RST_1;
    DEV_Delay_ms(100);
    PICO_LCD_1IN14_RST_0;
    DEV_Delay_ms(100);
    PICO_LCD_1IN14_RST_1;
    DEV_Delay_ms(100);
    LCD_CS_0;
}

/******************************************************************************
function :	send command
parameter:
     Reg : Command register
******************************************************************************/
static void PICO_LCD_1IN14_SendCommand(UBYTE Reg)
{
    PICO_LCD_1IN14_DC_0;
    // PICO_LCD_1IN14_CS_0;
    DEV_SPI_WriteByte(Reg);
    // PICO_LCD_1IN14_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
static void PICO_LCD_1IN14_SendData_8Bit(UBYTE Data)
{
    PICO_LCD_1IN14_DC_1;
    // PICO_LCD_1IN14_CS_0;
    DEV_SPI_WriteByte(Data);
    // PICO_LCD_1IN14_CS_1;
}

/******************************************************************************
function :	send data
parameter:
    Data : Write data
******************************************************************************/
static void PICO_LCD_1IN14_SendData_16Bit(UWORD Data)
{
    PICO_LCD_1IN14_DC_1;
    // PICO_LCD_1IN14_CS_0;
    DEV_SPI_WriteByte((Data >> 8) & 0xFF);
    DEV_SPI_WriteByte(Data & 0xFF);
    // PICO_LCD_1IN14_CS_1;
}

/******************************************************************************
function :	Initialize the lcd register
parameter:
******************************************************************************/
static void PICO_LCD_1IN14_InitReg(void)
{
    PICO_LCD_1IN14_SendCommand(0x3A);
    PICO_LCD_1IN14_SendData_8Bit(0x05);

    PICO_LCD_1IN14_SendCommand(0xB2);
    PICO_LCD_1IN14_SendData_8Bit(0x0C);
    PICO_LCD_1IN14_SendData_8Bit(0x0C);
    PICO_LCD_1IN14_SendData_8Bit(0x00);
    PICO_LCD_1IN14_SendData_8Bit(0x33);
    PICO_LCD_1IN14_SendData_8Bit(0x33);

    PICO_LCD_1IN14_SendCommand(0xB7);  //Gate Control
    PICO_LCD_1IN14_SendData_8Bit(0x35);

    PICO_LCD_1IN14_SendCommand(0xBB);  //VCOM Setting
    PICO_LCD_1IN14_SendData_8Bit(0x19);

    PICO_LCD_1IN14_SendCommand(0xC0); //LCM Control     
    PICO_LCD_1IN14_SendData_8Bit(0x2C);

    PICO_LCD_1IN14_SendCommand(0xC2);  //VDV and VRH Command Enable
    PICO_LCD_1IN14_SendData_8Bit(0x01);
    PICO_LCD_1IN14_SendCommand(0xC3);  //VRH Set
    PICO_LCD_1IN14_SendData_8Bit(0x12);
    PICO_LCD_1IN14_SendCommand(0xC4);  //VDV Set
    PICO_LCD_1IN14_SendData_8Bit(0x20);

    PICO_LCD_1IN14_SendCommand(0xC6);  //Frame Rate Control in Normal Mode
    PICO_LCD_1IN14_SendData_8Bit(0x0F);
    
    PICO_LCD_1IN14_SendCommand(0xD0);  // Power Control 1
    PICO_LCD_1IN14_SendData_8Bit(0xA4);
    PICO_LCD_1IN14_SendData_8Bit(0xA1);

    PICO_LCD_1IN14_SendCommand(0xE0);  //Positive Voltage Gamma Control
    PICO_LCD_1IN14_SendData_8Bit(0xD0);
    PICO_LCD_1IN14_SendData_8Bit(0x04);
    PICO_LCD_1IN14_SendData_8Bit(0x0D);
    PICO_LCD_1IN14_SendData_8Bit(0x11);
    PICO_LCD_1IN14_SendData_8Bit(0x13);
    PICO_LCD_1IN14_SendData_8Bit(0x2B);
    PICO_LCD_1IN14_SendData_8Bit(0x3F);
    PICO_LCD_1IN14_SendData_8Bit(0x54);
    PICO_LCD_1IN14_SendData_8Bit(0x4C);
    PICO_LCD_1IN14_SendData_8Bit(0x18);
    PICO_LCD_1IN14_SendData_8Bit(0x0D);
    PICO_LCD_1IN14_SendData_8Bit(0x0B);
    PICO_LCD_1IN14_SendData_8Bit(0x1F);
    PICO_LCD_1IN14_SendData_8Bit(0x23);

    PICO_LCD_1IN14_SendCommand(0xE1);  //Negative Voltage Gamma Control
    PICO_LCD_1IN14_SendData_8Bit(0xD0);
    PICO_LCD_1IN14_SendData_8Bit(0x04);
    PICO_LCD_1IN14_SendData_8Bit(0x0C);
    PICO_LCD_1IN14_SendData_8Bit(0x11);
    PICO_LCD_1IN14_SendData_8Bit(0x13);
    PICO_LCD_1IN14_SendData_8Bit(0x2C);
    PICO_LCD_1IN14_SendData_8Bit(0x3F);
    PICO_LCD_1IN14_SendData_8Bit(0x44);
    PICO_LCD_1IN14_SendData_8Bit(0x51);
    PICO_LCD_1IN14_SendData_8Bit(0x2F);
    PICO_LCD_1IN14_SendData_8Bit(0x1F);
    PICO_LCD_1IN14_SendData_8Bit(0x1F);
    PICO_LCD_1IN14_SendData_8Bit(0x20);
    PICO_LCD_1IN14_SendData_8Bit(0x23);

    PICO_LCD_1IN14_SendCommand(0x21);  //Display Inversion On

    PICO_LCD_1IN14_SendCommand(0x11);  //Sleep Out

    PICO_LCD_1IN14_SendCommand(0x29);  //Display On
}

/********************************************************************************
function:	Set the resolution and scanning method of the screen
parameter:
		Scan_dir:   Scan direction
********************************************************************************/
static void PICO_LCD_1IN14_SetAttributes(UBYTE Scan_dir)
{
    //Get the screen scan direction
    PICO_LCD_1IN14.SCAN_DIR = Scan_dir;
    UBYTE MemoryAccessReg = 0x00;

    //Get GRAM and LCD width and height
    if(Scan_dir == HORIZONTAL) {
        PICO_LCD_1IN14.HEIGHT	= PICO_LCD_1IN14_WIDTH;
        PICO_LCD_1IN14.WIDTH   = PICO_LCD_1IN14_HEIGHT;
        MemoryAccessReg = 0X70;
    } else {
        PICO_LCD_1IN14.HEIGHT	= PICO_LCD_1IN14_HEIGHT;       
        PICO_LCD_1IN14.WIDTH   = PICO_LCD_1IN14_WIDTH;
        MemoryAccessReg = 0X00;
    }

    // Set the read / write scan direction of the frame memory
    PICO_LCD_1IN14_SendCommand(0x36); //MX, MY, RGB mode
    PICO_LCD_1IN14_SendData_8Bit(MemoryAccessReg);	//0x08 set RGB
}

/********************************************************************************
function :	Initialize the lcd
parameter:
********************************************************************************/
void PICO_LCD_1IN14_Init(UBYTE Scan_dir)
{
    //Turn on the backlight
    PICO_LCD_1IN14_BL_1;

    //Hardware reset
    PICO_LCD_1IN14_Reset();

    //Set the resolution and scanning method of the screen
    PICO_LCD_1IN14_SetAttributes(Scan_dir);
    
    //Set the initialization register
    PICO_LCD_1IN14_InitReg();
}

/********************************************************************************
function:	Sets the start position and size of the display area
parameter:
		Xstart 	:   X direction Start coordinates
		Ystart  :   Y direction Start coordinates
		Xend    :   X direction end coordinates
		Yend    :   Y direction end coordinates
********************************************************************************/
void PICO_LCD_1IN14_SetWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend)
{
    UBYTE x,y;
    if(PICO_LCD_1IN14.SCAN_DIR == HORIZONTAL){x=40;y=53;}
    else{ x=52; y=40; }
    //set the X coordinates
    PICO_LCD_1IN14_SendCommand(0x2A);
    
    
    PICO_LCD_1IN14_SendData_16Bit(Xstart	+x);
    PICO_LCD_1IN14_SendData_16Bit(Xend-1	+x);
    //set the Y coordinates
    PICO_LCD_1IN14_SendCommand(0x2B);
    PICO_LCD_1IN14_SendData_16Bit(Ystart +y);
    PICO_LCD_1IN14_SendData_16Bit(Yend-1	  +y);

    PICO_LCD_1IN14_SendCommand(0X2C);
    //printf("%d %d\r\n",x,y);
}

/******************************************************************************
function :	Clear screen
parameter:
******************************************************************************/
void PICO_LCD_1IN14_Clear(UWORD Color)
{
    UWORD j;
    UWORD Image[PICO_LCD_1IN14.WIDTH*PICO_LCD_1IN14.HEIGHT];
    
    Color = ((Color<<8)&0xff00)|(Color>>8);
   
    for (j = 0; j < PICO_LCD_1IN14.HEIGHT*PICO_LCD_1IN14.WIDTH; j++) {
        Image[j] = Color;
    }
    
    PICO_LCD_1IN14_SetWindows(0, 0, PICO_LCD_1IN14.WIDTH, PICO_LCD_1IN14.HEIGHT);
    PICO_LCD_1IN14_DC_1;
    for(j = 0; j < PICO_LCD_1IN14.HEIGHT; j++){
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*PICO_LCD_1IN14.WIDTH], PICO_LCD_1IN14.WIDTH*2);
    }
}

/******************************************************************************
function :	Sends the image buffer in RAM to displays
parameter:
******************************************************************************/
void PICO_LCD_1IN14_Display(UWORD *Image)
{
    UWORD j;
    PICO_LCD_1IN14_SetWindows(0, 0, PICO_LCD_1IN14.WIDTH, PICO_LCD_1IN14.HEIGHT);
    PICO_LCD_1IN14_DC_1;
    for (j = 0; j < PICO_LCD_1IN14.HEIGHT; j++) {
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*PICO_LCD_1IN14.WIDTH], PICO_LCD_1IN14.WIDTH*2);
    }
}

void PICO_LCD_1IN14_DisplayWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend, UWORD *Image)
{
    // display
    UDOUBLE Addr = 0;

    UWORD j;
    PICO_LCD_1IN14_SetWindows(Xstart, Ystart, Xend , Yend);
    PICO_LCD_1IN14_DC_1;
    for (j = Ystart; j < Yend - 1; j++) {
        Addr = Xstart + j * PICO_LCD_1IN14.WIDTH ;
        DEV_SPI_Write_nByte((uint8_t *)&Image[Addr], (Xend-Xstart)*2);
    }
}

void PICO_LCD_1IN14_DisplayPoint(UWORD X, UWORD Y, UWORD Color)
{
    PICO_LCD_1IN14_SetWindows(X,Y,X,Y);
    PICO_LCD_1IN14_SendData_16Bit(Color);
}

void  Handler_1IN14_LCD(int signo)
{
    //System Exit
    printf("\r\nHandler:Program stop\r\n");     
    DEV_ModuleExit();
	exit(0);
}
