
/***********************************************************************************************************************
* | file      	:	Pico_LCD_1in8.c
* |	version		:	V1.0
* | date		:	2023-09-03
* | function	:	On the ST7735S chip driver and clear screen, drawing lines, drawing, writing
					and other functions to achieve
***********************************************************************************************************************/

#include "Pico_LCD_1in8.h"
#include "DEV_Config.h"

#include <stdlib.h>		//itoa()
#include <stdio.h>

PICO_LCD_1IN8_DIS sPICO_LCD_1IN8_DIS;

/*******************************************************************************
function:
			Hardware reset
*******************************************************************************/
static void PICO_LCD_1IN8_Reset(void)
{
    PICO_LCD_1IN8_RST_1;
    DEV_Delay_ms(100);
    PICO_LCD_1IN8_RST_0;
    DEV_Delay_ms(100);
    PICO_LCD_1IN8_RST_1;
    DEV_Delay_ms(100);
}

/*******************************************************************************
function:
		Write register address and data
*******************************************************************************/
static void PICO_LCD_1IN8_WriteReg(uint8_t Reg)
{
    PICO_LCD_1IN8_DC_0;
    //PICO_LCD_1IN8_CS_0;
    DEV_SPI_WriteByte(Reg);
    //PICO_LCD_1IN8_CS_1;
}

static void PICO_LCD_1IN8_WriteData_8Bit(uint8_t Data)
{
    PICO_LCD_1IN8_DC_1;
    //PICO_LCD_1IN8_CS_0;
    DEV_SPI_WriteByte(Data);
    //PICO_LCD_1IN8_CS_1;
}
/*
static void PICO_LCD_1IN8_WriteData_16Bit(uint16_t Data)
{
    PICO_LCD_1IN8_DC_1;
    //PICO_LCD_1IN8_CS_0;
    DEV_SPI_WriteByte(Data >> 8);
    DEV_SPI_WriteByte(Data & 0XFF);
    //PICO_LCD_1IN8_CS_1;
}*/

static void PICO_LCD_1IN8_WriteData_NLen16Bit(uint16_t Data,uint32_t DataLen)
{
    uint32_t i;
    PICO_LCD_1IN8_DC_1;
    //PICO_LCD_1IN8_CS_0;
    for(i = 0; i < DataLen; i++) {
        DEV_SPI_WriteByte( (uint8_t)(Data >> 8) );
        DEV_SPI_WriteByte( (uint8_t)(Data & 0XFF) );
    }
    //PICO_LCD_1IN8_CS_1;
}

/*******************************************************************************
function:
		Common register initialization
*******************************************************************************/
static void PICO_LCD_1IN8_InitReg(void)
{
    //ST7735R Frame Rate
    PICO_LCD_1IN8_WriteReg(0xB1);
    PICO_LCD_1IN8_WriteData_8Bit(0x01);
    PICO_LCD_1IN8_WriteData_8Bit(0x2C);
    PICO_LCD_1IN8_WriteData_8Bit(0x2D);

    PICO_LCD_1IN8_WriteReg(0xB2);
    PICO_LCD_1IN8_WriteData_8Bit(0x01);
    PICO_LCD_1IN8_WriteData_8Bit(0x2C);
    PICO_LCD_1IN8_WriteData_8Bit(0x2D);

    PICO_LCD_1IN8_WriteReg(0xB3);
    PICO_LCD_1IN8_WriteData_8Bit(0x01);
    PICO_LCD_1IN8_WriteData_8Bit(0x2C);
    PICO_LCD_1IN8_WriteData_8Bit(0x2D);
    PICO_LCD_1IN8_WriteData_8Bit(0x01);
    PICO_LCD_1IN8_WriteData_8Bit(0x2C);
    PICO_LCD_1IN8_WriteData_8Bit(0x2D);

    PICO_LCD_1IN8_WriteReg(0xB4); //Column inversion
    PICO_LCD_1IN8_WriteData_8Bit(0x07);

    //ST7735R Power Sequence
    PICO_LCD_1IN8_WriteReg(0xC0);
    PICO_LCD_1IN8_WriteData_8Bit(0xA2);
    PICO_LCD_1IN8_WriteData_8Bit(0x02);
    PICO_LCD_1IN8_WriteData_8Bit(0x84);
    PICO_LCD_1IN8_WriteReg(0xC1);
    PICO_LCD_1IN8_WriteData_8Bit(0xC5);

    PICO_LCD_1IN8_WriteReg(0xC2);
    PICO_LCD_1IN8_WriteData_8Bit(0x0A);
    PICO_LCD_1IN8_WriteData_8Bit(0x00);

    PICO_LCD_1IN8_WriteReg(0xC3);
    PICO_LCD_1IN8_WriteData_8Bit(0x8A);
    PICO_LCD_1IN8_WriteData_8Bit(0x2A);
    PICO_LCD_1IN8_WriteReg(0xC4);
    PICO_LCD_1IN8_WriteData_8Bit(0x8A);
    PICO_LCD_1IN8_WriteData_8Bit(0xEE);

    PICO_LCD_1IN8_WriteReg(0xC5); //VCOM
    PICO_LCD_1IN8_WriteData_8Bit(0x0E);

    //ST7735R Gamma Sequence
    PICO_LCD_1IN8_WriteReg(0xe0);
    PICO_LCD_1IN8_WriteData_8Bit(0x0f);
    PICO_LCD_1IN8_WriteData_8Bit(0x1a);
    PICO_LCD_1IN8_WriteData_8Bit(0x0f);
    PICO_LCD_1IN8_WriteData_8Bit(0x18);
    PICO_LCD_1IN8_WriteData_8Bit(0x2f);
    PICO_LCD_1IN8_WriteData_8Bit(0x28);
    PICO_LCD_1IN8_WriteData_8Bit(0x20);
    PICO_LCD_1IN8_WriteData_8Bit(0x22);
    PICO_LCD_1IN8_WriteData_8Bit(0x1f);
    PICO_LCD_1IN8_WriteData_8Bit(0x1b);
    PICO_LCD_1IN8_WriteData_8Bit(0x23);
    PICO_LCD_1IN8_WriteData_8Bit(0x37);
    PICO_LCD_1IN8_WriteData_8Bit(0x00);
    PICO_LCD_1IN8_WriteData_8Bit(0x07);
    PICO_LCD_1IN8_WriteData_8Bit(0x02);
    PICO_LCD_1IN8_WriteData_8Bit(0x10);

    PICO_LCD_1IN8_WriteReg(0xe1);
    PICO_LCD_1IN8_WriteData_8Bit(0x0f);
    PICO_LCD_1IN8_WriteData_8Bit(0x1b);
    PICO_LCD_1IN8_WriteData_8Bit(0x0f);
    PICO_LCD_1IN8_WriteData_8Bit(0x17);
    PICO_LCD_1IN8_WriteData_8Bit(0x33);
    PICO_LCD_1IN8_WriteData_8Bit(0x2c);
    PICO_LCD_1IN8_WriteData_8Bit(0x29);
    PICO_LCD_1IN8_WriteData_8Bit(0x2e);
    PICO_LCD_1IN8_WriteData_8Bit(0x30);
    PICO_LCD_1IN8_WriteData_8Bit(0x30);
    PICO_LCD_1IN8_WriteData_8Bit(0x39);
    PICO_LCD_1IN8_WriteData_8Bit(0x3f);
    PICO_LCD_1IN8_WriteData_8Bit(0x00);
    PICO_LCD_1IN8_WriteData_8Bit(0x07);
    PICO_LCD_1IN8_WriteData_8Bit(0x03);
    PICO_LCD_1IN8_WriteData_8Bit(0x10);

    PICO_LCD_1IN8_WriteReg(0xF0); //Enable test command
    PICO_LCD_1IN8_WriteData_8Bit(0x01);

    PICO_LCD_1IN8_WriteReg(0xF6); //Disable ram power save mode
    PICO_LCD_1IN8_WriteData_8Bit(0x00);

    PICO_LCD_1IN8_WriteReg(0x3A); //65k mode
    PICO_LCD_1IN8_WriteData_8Bit(0x05);

}


/********************************************************************************
function:	Set the display scan and color transfer modes
parameter:
		Scan_dir   :   Scan direction
		Colorchose :   RGB or GBR color format
********************************************************************************/
void PICO_LCD_1IN8_SetGramScanWay(PICO_LCD_1IN8_SCAN_DIR Scan_dir)
{
    //Get the screen scan direction
    sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Scan_Dir = Scan_dir;

	//Get GRAM and LCD width and height
	if(Scan_dir == L2R_U2D || Scan_dir == L2R_D2U || Scan_dir == R2L_U2D || Scan_dir == R2L_D2U){
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Dis_Column	= PICO_LCD_1IN8_HEIGHT ;
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Dis_Page = PICO_LCD_1IN8_WIDTH ;		
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_X_Adjust = PICO_LCD_1IN8_X;
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Y_Adjust = PICO_LCD_1IN8_Y;
	}else{
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Dis_Column	= PICO_LCD_1IN8_WIDTH ;
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Dis_Page = PICO_LCD_1IN8_HEIGHT ;	
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_X_Adjust = PICO_LCD_1IN8_Y;
		sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Y_Adjust = PICO_LCD_1IN8_X;
	}

    // Gets the scan direction of GRAM
    uint16_t MemoryAccessReg_Data=0;  //0x36
    switch (Scan_dir) {
    case L2R_U2D:
        MemoryAccessReg_Data = 0X00 | 0x00;//x Scan direction | y Scan direction
        break;
    case L2R_D2U:
        MemoryAccessReg_Data = 0x00 | 0x80;//0xC8 | 0X10
        break;
    case R2L_U2D:	//0X04
        MemoryAccessReg_Data = 0x40 | 0x00;
        break;
    case R2L_D2U:	//0X0C
        MemoryAccessReg_Data = 0x40 | 0x80;
        break;
    case U2D_L2R:	//0X02
        MemoryAccessReg_Data = 0X00 | 0X00 | 0x20;
        break;
    case U2D_R2L:	//0X06
        MemoryAccessReg_Data = 0x00 | 0X40 | 0x20;
        break;
    case D2U_L2R:	//0X0A
        MemoryAccessReg_Data = 0x80 | 0x00 | 0x20;
        break;
    case D2U_R2L:	//0X0E
        MemoryAccessReg_Data = 0x40 | 0x80 | 0x20;
        break;
    }

    // Set the read / write scan direction of the frame memory
    PICO_LCD_1IN8_WriteReg(0x36); //MX, MY, RGB mode
    PICO_LCD_1IN8_WriteData_8Bit( MemoryAccessReg_Data & 0xf7);	//RGB color filter panel

}

/********************************************************************************
function:
			initialization
********************************************************************************/
void PICO_LCD_1IN8_Init( PICO_LCD_1IN8_SCAN_DIR PICO_LCD_1IN8_ScanDir )
{
    //Turn on the backlight
    PICO_LCD_1IN8_BL_1;

    //Hardware reset
    PICO_LCD_1IN8_Reset();

    //Set the initialization register
    PICO_LCD_1IN8_InitReg();

    //Set the display scan and color transfer modes
    PICO_LCD_1IN8_SetGramScanWay( PICO_LCD_1IN8_ScanDir );
    DEV_Delay_ms(200);

    //sleep out
    PICO_LCD_1IN8_WriteReg(0x11);
    DEV_Delay_ms(120);

    //Turn on the LCD display
    PICO_LCD_1IN8_WriteReg(0x29);
}

/********************************************************************************
function:	Sets the start position and size of the display area
parameter:
		Xstart 	:   X direction Start coordinates
		Ystart  :   Y direction Start coordinates
		Xend    :   X direction end coordinates
		Yend    :   Y direction end coordinates
********************************************************************************/
void PICO_LCD_1IN8_SetWindows( POINT Xstart, POINT Ystart, POINT Xend, POINT Yend )
{

    //set the X coordinates
    PICO_LCD_1IN8_WriteReg ( 0x2A );
    PICO_LCD_1IN8_WriteData_8Bit ( 0x00 );						//Set the horizontal starting point to the high octet
    PICO_LCD_1IN8_WriteData_8Bit ( (Xstart & 0xff) + sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_X_Adjust);			//Set the horizontal starting point to the low octet
    PICO_LCD_1IN8_WriteData_8Bit ( 0x00 );				//Set the horizontal end to the high octet
    PICO_LCD_1IN8_WriteData_8Bit ( (( Xend ) & 0xff) + sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_X_Adjust);	//Set the horizontal end to the low octet

    //set the Y coordinates
    PICO_LCD_1IN8_WriteReg ( 0x2B );
    PICO_LCD_1IN8_WriteData_8Bit ( 0x00 );
    PICO_LCD_1IN8_WriteData_8Bit ( (Ystart & 0xff) + sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Y_Adjust);
    PICO_LCD_1IN8_WriteData_8Bit ( 0x00 );
    PICO_LCD_1IN8_WriteData_8Bit ( ( (Yend) & 0xff )+ sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Y_Adjust);

    PICO_LCD_1IN8_WriteReg(0x2C);

}

/********************************************************************************
function:	Set the display point (Xpoint, Ypoint)
parameter:
		xStart :   X direction Start coordinates
		xEnd   :   X direction end coordinates
********************************************************************************/
void PICO_LCD_1IN8_SetCursor ( POINT Xpoint, POINT Ypoint )
{
    PICO_LCD_1IN8_SetWindows ( Xpoint, Ypoint, Xpoint , Ypoint );
}

/********************************************************************************
function:	Set show color
parameter:
		Color  :   Set show color
********************************************************************************/
//static void PICO_LCD_1IN8_SetColor( LENGTH Dis_Width, LENGTH Dis_Height, COLOR Color ){
void PICO_LCD_1IN8_SetColor( COLOR Color ,POINT Xpoint, POINT Ypoint)
{
    PICO_LCD_1IN8_WriteData_NLen16Bit(Color ,(uint32_t)Xpoint * (uint32_t)Ypoint);
}

/********************************************************************************
function:	Point (Xpoint, Ypoint) Fill the color
parameter:
		Xpoint :   The x coordinate of the point
		Ypoint :   The y coordinate of the point
		Color  :   Set the color
********************************************************************************/
void PICO_LCD_1IN8_SetPointlColor ( POINT Xpoint, POINT Ypoint, COLOR Color )
{
    if ( ( Xpoint <= sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Dis_Column ) && ( Ypoint <= sPICO_LCD_1IN8_DIS.PICO_LCD_1IN8_Dis_Page ) ) {
        PICO_LCD_1IN8_SetCursor (Xpoint, Ypoint);
        PICO_LCD_1IN8_SetColor ( Color , 1 , 1);
    }
}

/********************************************************************************
function:	Fill the area with the color
parameter:
		Xstart :   Start point x coordinate
		Ystart :   Start point y coordinate
		Xend   :   End point coordinates
		Yend   :   End point coordinates
		Color  :   Set the color
********************************************************************************/
void PICO_LCD_1IN8_SetArealColor (POINT Xstart, POINT Ystart, POINT Xend, POINT Yend,	COLOR  Color)
{
    if((Xend > Xstart) && (Yend > Ystart)) {
        PICO_LCD_1IN8_SetWindows( Xstart , Ystart , Xend -1, Yend -1 );
        PICO_LCD_1IN8_SetColor ( Color ,Xend - Xstart , Yend - Ystart );
    }
}

/********************************************************************************
function:
			Clear screen
********************************************************************************/
void PICO_LCD_1IN8_Clear(COLOR  Color)
{
    UWORD j;
    UWORD Image[PICO_LCD_1IN8_HEIGHT*PICO_LCD_1IN8_WIDTH];
    
    Color = ((Color<<8)&0xff00)|(Color>>8);
   
    for (j = 0; j < PICO_LCD_1IN8_HEIGHT*PICO_LCD_1IN8_WIDTH; j++) {
        Image[j] = Color;
    }
    
    PICO_LCD_1IN8_SetWindows(0, 0, PICO_LCD_1IN8_WIDTH,PICO_LCD_1IN8_HEIGHT);
    PICO_LCD_1IN8_DC_1;
    for(j = 0; j <PICO_LCD_1IN8_HEIGHT; j++){
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*PICO_LCD_1IN8_WIDTH], PICO_LCD_1IN8_WIDTH*2);
    }
}

void PICO_LCD_1IN8_Display(UWORD *Image)
{
    UWORD j;
    PICO_LCD_1IN8_SetWindows(0, 0, PICO_LCD_1IN8_WIDTH-1, PICO_LCD_1IN8_HEIGHT-1);
    PICO_LCD_1IN8_DC_1;
    for (j = 0; j < PICO_LCD_1IN8_HEIGHT; j++) {
        DEV_SPI_Write_nByte((uint8_t *)&Image[j*PICO_LCD_1IN8_WIDTH], PICO_LCD_1IN8_WIDTH*2);
    }
}

void PICO_LCD_1IN8_DisplayWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend, UWORD *Image)
{
    UDOUBLE Addr = 0;

    UWORD j;
    PICO_LCD_1IN8_SetWindows(Xstart, Ystart, Xend-1 , Yend-1);
    PICO_LCD_1IN8_DC_1;
    for (j = Ystart; j < Yend - 1; j++) {
        Addr = Xstart + j * PICO_LCD_1IN8_WIDTH ;
        DEV_SPI_Write_nByte((uint8_t *)&Image[Addr], (Xend-Xstart)*2);
    }
}

void  Handler_1IN8_LCD(int signo)
{
    //System Exit
    printf("\r\nHandler:Program stop\r\n");     
    DEV_ModuleExit();
	exit(0);
}

