/*****************************************************************************
* | File      	:   Pico_LCD_1in8.h
* | Author      :   Luckfox team
* | Function    :   Hardware underlying interface
* | Info        :
*                Used to shield the underlying layers of each master 
*                and enhance portability
*----------------
* |	This version:   V1.0
* | Date        :   2023-09-03
* | Info        :   Basic version
*
******************************************************************************/

#ifndef __PICO_LCD_1IN8__H
#define __PICO_LCD_1IN8__H

#include <stdint.h>
#include "DEV_Config.h"
#include "../Fonts/fonts.h"

#define	COLOR				uint16_t		//The variable type of the color (unsigned short) 
#define	POINT				uint16_t		//The type of coordinate (unsigned short) 
#define	LENGTH				uint16_t		//The type of coordinate (unsigned short) 

/********************************************************************************
function:
		Define the full screen height length of the display
********************************************************************************/


#define PICO_LCD_1IN8_WIDTH  160  //LCD width
#define PICO_LCD_1IN8_HEIGHT  128 //LCD height
#define PICO_LCD_1IN8_X	 2
#define PICO_LCD_1IN8_Y	 1

#define PICO_LCD_1IN8_SetBacklight(Value) DEV_SetBacklight(Value) 

#define PICO_LCD_1IN8_CS_0	LCD_CS_0	 
#define PICO_LCD_1IN8_CS_1	LCD_CS_1	
	                  
#define PICO_LCD_1IN8_RST_0	LCD_RST_0	
#define PICO_LCD_1IN8_RST_1	LCD_RST_1	
	                 
#define PICO_LCD_1IN8_DC_0	LCD_DC_0	
#define PICO_LCD_1IN8_DC_1	LCD_DC_1	
	                  
#define PICO_LCD_1IN8_BL_0	LCD_BL_0	
#define PICO_LCD_1IN8_BL_1	LCD_BL_1	


/********************************************************************************
function:
			scanning method
********************************************************************************/
typedef enum {
    L2R_U2D  = 0,				//The display interface is displayed , left to right, up to down
    L2R_D2U  ,
    R2L_U2D  ,
    R2L_D2U  ,

    U2D_L2R  ,
    U2D_R2L  ,
    D2U_L2R  ,
    D2U_R2L  ,
} PICO_LCD_1IN8_SCAN_DIR;
#define SCAN_DIR_DFT  U2D_R2L  	//Default scan direction = L2R_U2D

/********************************************************************************
function:
	Defines the total number of rows in the display area
********************************************************************************/

typedef struct {
    LENGTH 				PICO_LCD_1IN8_Dis_Column;	//COLUMN
    LENGTH 				PICO_LCD_1IN8_Dis_Page;		//PAGE
    PICO_LCD_1IN8_SCAN_DIR 	PICO_LCD_1IN8_Scan_Dir;
    POINT 				PICO_LCD_1IN8_X_Adjust;		//LCD x actual display position calibration
    POINT 				PICO_LCD_1IN8_Y_Adjust;		//LCD y actual display position calibration
} PICO_LCD_1IN8_DIS;


void PICO_LCD_1IN8_Init( PICO_LCD_1IN8_SCAN_DIR Lcd_ScanDir );
void PICO_LCD_1IN8_Clear(COLOR  Color);
void PICO_LCD_1IN8_Display(UWORD *Image);
void PICO_LCD_1IN8_DisplayWindows(UWORD Xstart, UWORD Ystart, UWORD Xend, UWORD Yend, UWORD *Image);
void PICO_LCD_1IN8_SetPointlColor ( POINT Xpoint, POINT Ypoint, COLOR Color );
void Handler_1IN8_LCD(int signo);


void PICO_LCD_1IN8_SetGramScanWay(PICO_LCD_1IN8_SCAN_DIR Scan_dir);
void PICO_LCD_1IN8_SetWindows( POINT Xstart, POINT Ystart, POINT Xend, POINT Yend );
void PICO_LCD_1IN8_SetCursor ( POINT Xpoint, POINT Ypoint );
void PICO_LCD_1IN8_SetColor( COLOR Color ,POINT Xpoint, POINT Ypoint);
void PICO_LCD_1IN8_SetArealColor ( POINT Xstart, POINT Ystart, POINT Xend, POINT Yend,COLOR  Color);


#endif
